﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Data.SqlClient;
using System.Data;
using Microsoft.SqlServer.Server;
using System.Reflection;


namespace WebSiteUsersChartCore
{
    public class WebSiteUsersChart
    {
        private Bitmap bitmap;
        private Graphics graphics;
        private int chartWidth;
        private int chartHeight;
        private int borderPadding;
        private int axisBottomPadding;
        private int axisHorizontalPadding;
        private int penWidth;
        private int barPadding;
        private int barBottomPadding;
        private int barDataFontSize;
        private List<string> xAxisLabels;
        private List<int> dataList;
        private string chartTitle;
        private string continent;
        private int month;
        private int year;

        public WebSiteUsersChart(int chartWidth, int chartHeight, string chartTitle, string continent, int month, int year)
        {
            this.chartHeight = chartHeight;
            this.chartWidth = chartWidth;
            this.chartTitle = chartTitle;
            this.continent = continent;
            this.month = month;
            this.year = year;


            this.bitmap = new Bitmap(this.chartWidth, this.chartHeight);
            this.graphics = Graphics.FromImage(this.bitmap);
            this.graphics.SmoothingMode = SmoothingMode.AntiAlias;
            this.graphics.Clear(Color.White);
            this.borderPadding = 55;
            this.axisBottomPadding = 30;
            this.axisHorizontalPadding = 10;
            this.penWidth = 2;
            this.barPadding = 20;
            this.barBottomPadding = 5;
            this.barDataFontSize = 10;
        }

        public Bitmap PlotChart()
        {
            this.GetChartData();

            // If there is no data, return
            if (this.dataList.Count == 0)
            {
                return null;
            }

            // Load Face image
            Image faceImage = this.GetEmbeddedImage("WebSiteUsersChartCore.Images.Face.png");

            // Draw area border
            Rectangle border = new Rectangle(this.penWidth,
                                             this.penWidth,
                                             this.chartWidth - this.penWidth * 2,
                                             this.chartHeight - this.penWidth * 2);
            this.graphics.DrawRectangle(new Pen(Color.Black, this.penWidth), border);

            // Fill background
            this.graphics.FillRectangle(new LinearGradientBrush(border, Color.FromArgb(255, 255, 255), Color.FromArgb(219, 231, 245), LinearGradientMode.Vertical), border);

            // Calculate chart area
            Rectangle chartArea = new Rectangle(this.penWidth,
                                             this.borderPadding + this.penWidth,
                                             this.chartWidth - this.penWidth * 2,
                                             this.chartHeight - this.borderPadding - this.penWidth * 2);

            // Draw Chart Titile
            DateTime dateTime = new DateTime(year, month, 15);
            string consolidatedChartTitle = String.Format("{0} ({1}) {2} {3}", chartTitle, continent, dateTime.ToString("MMMM"), year);
            this.graphics.DrawString(consolidatedChartTitle,
                                     new Font("Calibri",
                                     12,
                                     FontStyle.Bold),
                                     Brushes.Black,
                                     10, 10);

            // Draw x-axis
            this.graphics.DrawLine(new Pen(Color.Black, this.penWidth), chartArea.Left + this.axisHorizontalPadding, chartArea.Bottom - this.axisBottomPadding, chartArea.Right - this.axisHorizontalPadding, chartArea.Bottom - this.axisBottomPadding);
            int xAxisLength = (chartArea.Right - this.axisHorizontalPadding) - (chartArea.Left + this.axisHorizontalPadding);

            // Draw x-axis Labels
            int barCount = xAxisLabels.Count;
            int barWidth = xAxisLength / barCount;
            int nextLabelStartPoint = chartArea.Left + this.axisHorizontalPadding;
            foreach (string xAxisLabel in xAxisLabels)
            {
                int centerPoint = nextLabelStartPoint + barWidth / 2;
                float stringWidth = this.graphics.MeasureString(xAxisLabel, new Font("Calibri", 10, FontStyle.Bold)).Width;
                this.graphics.DrawString(xAxisLabel,
                                         new Font("Calibri", 10, FontStyle.Bold),
                                         Brushes.Black,
                                         centerPoint - stringWidth / 2,
                                         chartArea.Bottom - this.axisBottomPadding);

                nextLabelStartPoint += barWidth;
            }

            // Draw Bars
            int maxValue = dataList.Max();
            int minValue = dataList.Min();
            int nextBarStartPoint = chartArea.Left + this.axisHorizontalPadding;
            int maxBarHeight = chartArea.Bottom - this.axisBottomPadding - chartArea.Top;
            foreach (int data in dataList)
            {
                int barHeight = (data * maxBarHeight / maxValue) - this.barDataFontSize;
                int dataLabelPoint = nextBarStartPoint + barWidth / 2 - 10;

                // Fill bar
                Rectangle barFillArea = new Rectangle(nextBarStartPoint + barPadding, chartArea.Top + (maxBarHeight - barHeight), barWidth - 2 * barPadding, barHeight - this.barBottomPadding);
                this.graphics.FillRectangle(new LinearGradientBrush(barFillArea, Color.FromArgb(202, 234, 247), Color.FromArgb(0, 175, 220), LinearGradientMode.Vertical), barFillArea);

                // Draw data labels
                this.graphics.DrawString(data.ToString(), new Font("Calibri", this.barDataFontSize, FontStyle.Bold | FontStyle.Italic), Brushes.Black, dataLabelPoint, chartArea.Top + (maxBarHeight - barHeight) - this.barDataFontSize - 5);
                nextBarStartPoint += barWidth;

                // Draw Face image in bar
                this.graphics.DrawImage(faceImage, new Rectangle(barFillArea.X, barFillArea.Y, barWidth - 2 * barPadding, (barWidth - 2 * barPadding) * 85 / 73));
            }

            return this.bitmap;
        }

        private void GetChartData()
        {
            this.GetUserData();
        }

        private void GetUserData()
        {
            dataList = new List<int>();
            xAxisLabels = new List<string>();

            // Replace the connection string depending on wether testing using Test Harness or
            // deploying the solution to SQL Server as a SQL CLR function
            //string connectionString = "context connection=true";
            string connectionString = "Data Source=.;Initial Catalog=WebsiteUsersReportDB; Integrated Security=sspi;";

            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                using (SqlCommand sqlCommand = new SqlCommand("dbo.usp_GetUsageData", connection))
                {
                    sqlCommand.CommandType = CommandType.StoredProcedure;
                    sqlCommand.Parameters.Add("@Month", SqlDbType.Int).Value = this.month;
                    sqlCommand.Parameters.Add("@Year", SqlDbType.Int).Value = this.year;
                    sqlCommand.Parameters.Add("@Region", SqlDbType.NVarChar).Value = continent;
                    connection.Open();
                    SqlDataReader reader = sqlCommand.ExecuteReader();
                    while (reader.Read())
                    {
                        this.xAxisLabels.Add(reader.GetString(reader.GetOrdinal("CountryName")));
                        this.dataList.Add(reader.GetInt32(reader.GetOrdinal("UserCount")));

                    }
                }
            }
        }

        private Image GetEmbeddedImage(string name)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();
            Stream imageStream = assembly.GetManifestResourceStream(name);
            return Bitmap.FromStream(imageStream);
        }
    }
}
